// KaraluchSim.cpp : Plik biblioteczny Karalucha
//

extern "C" {
#include "extApi.h"
}

using namespace std;

//Klasa opisujca nog robota
class Noga {
	//Pooenie nogi we wspdnych robota [mm]
	struct {
		float x, y, z;
	}Baza;
	//Kty przegubw nogi [rad]
	float biodroH = 0, biodroV = 0, kolano = 0;
public:
public:
	//Stae wartoci definiujace przeguby
	enum Staw { BIODROH = 0, BIODROV = 1, KOLANO = 2 };
	//Identyfikator klienta symulacji robota (jedna wart da wszstkich ng)
	static int klientID;
	//Identyfikator pierwszego przegubu - biodra horyzontalnie
	int BiodroHID = 0;
	//Identyfikator drugiego przegubu - biodra wertykalnie
	int BiodroVID = 0;
	//Identyfikator trzeciego przegubu - kolana
	int KolanoID = 0;
	//Konstruktor klasy Noga
	Noga() { Noga(0, 0, 0); }
	//Kostruktor klasy Noga z moliwoci zdefiniowania pooenia Nogi we wsprzdnych robota
	//fX, fY, fZ [float] - wsprzdne mocowania nogi w robocie [mm]
	Noga(float fX, float fY, float fZ) { Baza.x = fX; Baza.y = fY; Baza.z = fZ; }
	//Przekazuje warto kta przegubu pierwszego BIODROH
	//zwraca [float] - kt przegubu pierwszego BIODROH [rad]
	float BiodroH() { return biodroH; }
	//Odczyt wartoci kta przegubu pierwszego roota BIODROH z moliwoci oczekiwania na wykonanie
	//bCzekaj [bool] - opcjonalne okrelenie czy ma oczekiwac na wykonanie polecenia (domylnie false)
	//zwraca [float] - Kt przegubu pierwszego robota BIODROH [rad]
	float GetBiodroH(bool bCzekaj = false) {
		simxFloat fKat = 0;
		simxGetJointPosition(klientID, BiodroHID, &fKat, (bCzekaj) ? simx_opmode_oneshot_wait : simx_opmode_oneshot);
		biodroH = fKat;
		return fKat;
	}
	//Zadanie wartoci kta przegubu pierwszego BIODROH z moliwoci oczekiwania na wykonanie
	//fKat [float[ - kt przegubu pierwszego robota BIODROH [rad]
	//bCzekaj [bool] - opcjonalne okrelenie czy ma oczekiwac na wykonanie polecenia (domylnie false)
	void SetBiodroH(float fKat, bool bCzekaj = false) {
		simxSetJointTargetPosition(klientID, BiodroHID, fKat, (bCzekaj) ? simx_opmode_oneshot_wait : simx_opmode_oneshot);
		biodroH = fKat;
	}
	//Przekazuje warto kta przegubu drugiego BIODROV
	//zwraca [float] - kt przegubu drugiego BIODROV [rad]
	float BiodroV() { return biodroV; }
	//Odczyt wartoci kta przegubu drugiego w robcie BIODROV z moliwoci oczekiwania na wykonanie
	//bCzekaj [bool] - opcjonalne okrelenie czy ma oczekiwac na wykonanie polecenia (domylnie false)
	//zwraca [float] - Kt przegubu drugiego robota BIODROV [rad]
	float GetBiodroV(bool bCzekaj = false) {
		simxFloat fKat = 0;
		simxGetJointPosition(klientID, BiodroVID, &fKat, (bCzekaj) ? simx_opmode_oneshot_wait : simx_opmode_oneshot);
		biodroV = fKat;
		return fKat;
	}
	//Zadanie wartoci kta przegubu drugiego BIODROV z moliwoci oczekiwania na wykonanie
	//fKat [float[ - kt przegubu drugiego robota BIODROV [rad]
	//bCzekaj [bool] - opcjonalne okrelenie czy ma oczekiwac na wykonanie polecenia (domylnie false)
	void SetBiodroV(float fKat, bool bCzekaj = false) {
		simxSetJointTargetPosition(klientID, BiodroVID, fKat, (bCzekaj) ? simx_opmode_oneshot_wait : simx_opmode_oneshot);
		biodroV = fKat;
	}
	//Przekazuje warto kta przegubu trzeciego KOLANO
	//zwraca [float] - kt przegubu trzeciego KOLANO [rad]
	float Kolano() { return kolano; }
	//Odczyt wartoci kta przegubu trzeciego w robcie KOLANO z moliwoci oczekiwania na wykonanie
	//bCzekaj [bool] - opcjonalne okrelenie czy ma oczekiwac na wykonanie polecenia (domylnie false)
	//zwraca [float] - Kt przegubu trzeciego robota KOLANO [rad]
	float GetKolano(bool bCzekaj = false) {
		simxFloat fWartosc = 0;
		simxGetJointPosition(klientID, KolanoID, &fWartosc, (bCzekaj) ? simx_opmode_oneshot_wait : simx_opmode_oneshot);
		kolano = fWartosc;
		return fWartosc;
	}
	//Zadanie wartoci kta przegubu trzeciego KOLANO z moliwoci oczekiwania na wykonanie
	//fKat [float[ - kt przegubu trzeciego robota KOLANO [rad]
	//bCzekaj [bool] - opcjonalne okrelenie czy ma oczekiwac na wykonanie polecenia (domylnie false)
	void SetKolano(float fKat, bool bCzekaj = false) {
		simxSetJointTargetPosition(klientID, KolanoID, fKat, (bCzekaj) ? simx_opmode_oneshot_wait : simx_opmode_oneshot);
		kolano = fKat;
	}
	//Odczyt momentu przegubu nogi robota z moliwoci oczekiwania na wykoananie
	//stStaw [Staw] - wybrany staw / przegub [Staw]
	//bCzekaj [bool] - opcjonalne okrelenie czy ma oczekiwac na wykonanie polecenia (domylnie false)
	//zwraca [float] - warto momentu [Nm]
	float GetMoment(Staw stStaw, bool bCzekaj = false) {
		float fMoment = 0;
		switch (stStaw) {
		case Staw::BIODROH:
			simxGetJointForce(klientID, BiodroHID, &fMoment, (bCzekaj) ? simx_opmode_oneshot_wait : simx_opmode_oneshot_split);
			break;
		case Staw::BIODROV:
			simxGetJointForce(klientID, BiodroVID, &fMoment, (bCzekaj) ? simx_opmode_oneshot_wait : simx_opmode_oneshot_split);
			break;
		case Staw::KOLANO:
			simxGetJointForce(klientID, KolanoID, &fMoment, (bCzekaj) ? simx_opmode_oneshot_wait : simx_opmode_oneshot_split);
			break;
		}
		return fMoment;
	}
	//Ustawienie momentu przegubu nogi robota z moliwoci oczekiwania na wykoananie
	//stStaw [Staw] - wybrany staw / przegub [Staw]
	//fMoment [float] - nowa warto momentu dla wybranego stawu [Nm]
	//bCzekaj [bool] - opcjonalne okrelenie czy ma oczekiwac na wykonanie polecenia (domylnie false)
	void SetMoment(Staw stStaw, float fMoment, bool bCzekaj = false) {
		switch (stStaw) {
		case Staw::BIODROH:
			simxSetJointForce(klientID, BiodroHID, fMoment, (bCzekaj) ? simx_opmode_oneshot_wait : simx_opmode_oneshot_split);
			break;
		case Staw::BIODROV:
			simxSetJointForce(klientID, BiodroVID, fMoment, (bCzekaj) ? simx_opmode_oneshot_wait : simx_opmode_oneshot_split);
			break;
		case Staw::KOLANO:
			simxSetJointForce(klientID, KolanoID, fMoment, (bCzekaj) ? simx_opmode_oneshot_wait : simx_opmode_oneshot_split);
			break;
		}
	}
	//Odczyt momentw przegubw nogi robota z moliwoci oczekiwania na wykoananie
	//fMomenty [float[3]] - tablica nowych wartoci momentw dla przegubw nogi robota: BiodroH, BiodroV i Kolano [Nm]
	//bCzekaj [bool] - opcjonalne okrelenie czy ma oczekiwac na wykonanie polecenia (domylnie false)
	void GetMomenty(float fMomenty[3], bool bCzekaj = false) {
		fMomenty[0] = GetMoment(Staw::BIODROH, bCzekaj);
		fMomenty[1] = GetMoment(Staw::BIODROV, bCzekaj);
		fMomenty[2] = GetMoment(Staw::KOLANO, bCzekaj);
	}
	//Ustawienie momentu przegubw nogi robota z moliwoci oczekiwania na wykoananie
	//fMoment [float] - nowa warto momentu dla wszystkich przegubw nogi [Nm]
	//bCzekaj [bool] - opcjonalne okrelenie czy ma oczekiwac na wykonanie polecenia (domylnie false)
	void SetMomenty(float fMoment, bool bCzekaj = false) {
		SetMoment(Staw::BIODROH, fMoment, bCzekaj);
		SetMoment(Staw::BIODROV, fMoment, bCzekaj);
		SetMoment(Staw::KOLANO, fMoment, bCzekaj);
	}
	//Odczyt maksymalnego momentu przegubu nogi robota z moliwoci oczekiwania na wykoananie
	//stStaw [Staw] - wybrany staw / przegub [Staw]
	//bCzekaj [bool] - opcjonalne okrelenie czy ma oczekiwac na wykonanie polecenia (domylnie false)
	//zwraca [float] - warto maksymalnego momentu [Nm]
	float GetMomentMAX(Staw stStaw, bool bCzekaj = false) {
		float fMomentMax = 0;
		switch (stStaw)
		{
		case BIODROH:
			simxGetJointMaxForce(klientID, BiodroHID, &fMomentMax, (bCzekaj) ? simx_opmode_oneshot_wait : simx_opmode_oneshot_split);
			break;
		case BIODROV:
			simxGetJointMaxForce(klientID, BiodroVID, &fMomentMax, (bCzekaj) ? simx_opmode_oneshot_wait : simx_opmode_oneshot_split);
			break;
		case KOLANO:
			simxGetJointMaxForce(klientID, KolanoID, &fMomentMax, (bCzekaj) ? simx_opmode_oneshot_wait : simx_opmode_oneshot_split);
			break;
		}
		return fMomentMax;
	}
	//Ustawienie maksymalnego momentu przegubw nogi robota z moliwoci oczekiwania na wykoananie
	//stStaw [Staw] - wybrany staw / przegub [Staw]
	//FMomentMax [float] - nowa warto maksymalnego momentu [Nm]
	//bCzekaj [bool] - opcjonalne okrelenie czy ma oczekiwac na wykonanie polecenia (domylnie false)
	void SetMomentMAX(Staw stStaw, float fMomentMax, bool bCzekaj = false) {
		switch (stStaw)
		{
		case BIODROH:
			simxSetJointMaxForce(klientID, BiodroHID, fMomentMax, (bCzekaj) ? simx_opmode_oneshot_wait : simx_opmode_oneshot_split);
			break;
		case BIODROV:
			simxSetJointMaxForce(klientID, BiodroVID, fMomentMax, (bCzekaj) ? simx_opmode_oneshot_wait : simx_opmode_oneshot_split);
			break;
		case KOLANO:
			simxSetJointMaxForce(klientID, KolanoID, fMomentMax, (bCzekaj) ? simx_opmode_oneshot_wait : simx_opmode_oneshot_split);
			break;
		}
	}
	//Odczyt maksymalnych momentw przegubw nogi robota z moliwoci oczekiwania na wykoananie
	//fMomentyMax [float[3]] - tablica wartoci momentw maksymalnych dla przegubw nogi BiodroH, BiodroV, Kolano [Nm]
	//bCzekaj [bool] - opcjonalne okrelenie czy ma oczekiwac na wykonanie polecenia (domylnie false)
	void GetMomentyMAX(float fMomentyMax[3], bool bCzekaj = false) {
		fMomentyMax[0] = GetMomentMAX(BIODROH, bCzekaj);
		fMomentyMax[1] = GetMomentMAX(BIODROV, bCzekaj);
		fMomentyMax[2] = GetMomentMAX(KOLANO, bCzekaj);
	}
	//Ustawienie maksymalnego momentu przegubw nogi robota z moliwoci oczekiwania na wykoananie
	//fMomentyMax [float] - warto momentu maksymalnego dla przegubw nogi BiodroH, BiodroV, Kolano [Nm]
	//bCzekaj [bool] - opcjonalne okrelenie czy ma oczekiwac na wykonanie polecenia (domylnie false)
	void SetMomentyMAX(float fMomentMax, bool bCzekaj = false) {
		SetMomentMAX(BIODROH, fMomentMax, bCzekaj);
		SetMomentMAX(BIODROV, fMomentMax, bCzekaj);
		SetMomentMAX(KOLANO, fMomentMax, bCzekaj);
	}
	//UStawienie prdkoci przegubu nogi robota z moliwoci oczekiwania na wykonanie
	//stStaw [Staw] - wybrany staw / przegub [Staw]
	//fPredkosc [float] - nwowa warto prdkoci [st/s]
	//bCzekaj [bool] - opcjonalne okrelenie czy ma oczekiwac na wykonanie polecenia (domylnie false)
	float SetPredkosc(Staw stStaw, float fPredkosc, bool bCzekaj = false) {
		switch (stStaw)
		{
		case BIODROH:
			simxSetJointTargetVelocity(klientID, BiodroHID, fPredkosc, (bCzekaj) ? simx_opmode_oneshot_wait : simx_opmode_oneshot_split);
			break;
		case BIODROV:
			simxSetJointTargetVelocity(klientID, BiodroVID, fPredkosc, (bCzekaj) ? simx_opmode_oneshot_wait : simx_opmode_oneshot_split);
			break;
		case KOLANO:
			simxSetJointTargetVelocity(klientID, KolanoID, fPredkosc, (bCzekaj) ? simx_opmode_oneshot_wait : simx_opmode_oneshot_split);
			break;
		}
	}
	//UStawienie prdkoci przegubw nogi robota z moliwoci oczekiwania na wykonanie
	//fPredkosc [float] - nwowa warto prdkoci [st/s]
	//bCzekaj [bool] - opcjonalne okrelenie czy ma oczekiwac na wykonanie polecenia (domylnie false)
	void SetPredkosci(float fPredkosc, bool bCzekaj = false) {
		SetPredkosc(BIODROH, fPredkosc, bCzekaj);
		SetPredkosc(BIODROV, fPredkosc, bCzekaj);
		SetPredkosc(KOLANO, fPredkosc, bCzekaj);
	}
	//Zadanie wartoci ktw przegubw robota
	//fBiodroH, fBiodroV, fKolano [float] - kty przegubw nogi robota [rad]
	void SetKatyNogi(float fBiodroH, float fBiodroV, float fKolano) {
		SetBiodroH(fBiodroH);
		SetBiodroV(fBiodroV);
		SetKolano(fKolano);
	}
	//Odczyt wartoci ktw przegubw robota
	//fBSK [float[3]] - kty przegubw nogi robota [rad]
	void GetKatyNogi(float fBKS[3]) {
		GetBiodroH(); GetBiodroV(); GetKolano();
		fBKS[0] = biodroH;
		fBKS[1] = biodroV;
		fBKS[2] = kolano;
	}
	//Ustawienie pozycji kocwki nogi robota we wsprzdnych robota
	//fX, fY, fZ [float] - pozycja nogi robota [mm]
	void SetPozycjaNogiAtRobot(float fX, float fY, float fZ) { SetPozycjaNogiAtNoga(fX - Baza.x, fY - Baza.y, fZ - Baza.z); }
	//Odczyt pozycji nogi robota we wsprzdnych nogi z moliwoci oczekiwania na wykonanie
	//fXYZ [float[3]] - wsrzdne XYZ [mm]
	//bCzekaj [bool] - opcjonalne okrelenie czy ma oczekiwac na wykonanie polecenia (domylnie false)
	void GetPozycjaNogiAtNoga(float fXYZ[3], bool bCzekaj = false) {
		GetBiodroH(bCzekaj); GetBiodroV(bCzekaj); GetKolano(bCzekaj);
		if (Baza.y < 0) {
			fXYZ[0] = (25 + 60 * cos(biodroV) - 120 * sin(kolano + biodroV)) * sin(biodroH);
			fXYZ[1] = -(25 + 60 * cos(biodroV) - 120 * sin(kolano + biodroV)) * cos(biodroH);
			fXYZ[2] = -60 * sin(biodroV) - 120 * cos(kolano + biodroV);
		}
		else {
			fXYZ[0] = (25 + 60 * cos(biodroV) + 120 * sin(kolano - biodroV)) * sin(-biodroH);
			fXYZ[1] = (25 + 60 * cos(biodroV) + 120 * sin(kolano - biodroV)) * cos(biodroH);
			fXYZ[2] = -60 * sin(biodroV) - 120 * cos(kolano - biodroV);
		}
	}
	//Ustawienie pozycji kocwki nogi robota we wsprzdnych nogi z moliwoci oczekiwania na wykonanie
	//fX, fY, fZ [float] - wsprzne XYZ [mm]
	//bCzekaj [bool] - opcjonalne okrelenie czy ma oczekiwac na wykonanie polecenia (domylnie false)
	//zwraca [int] - 0 gdy OK, -1 gdy bd
	int SetPozycjaNogiAtNoga(float fX, float fY, float fZ, bool bCzekaj = false) {
		if (Baza.y < 0) {	//lewa noga
			if (fY < 0) {
				SetBiodroH(atan2(fX, -fY), bCzekaj);
				SetBiodroV(atan((sqrt(fX * fX + fY * fY) - 25) / fZ) - acos((-10800 + fZ * fZ + pow(sqrt(fX * fX + fY * fY) - 25, 2)) / 120 / sqrt(fZ * fZ + pow(sqrt(fX * fX + fY * fY) - 25, 2))) + 1.57, bCzekaj);
				SetKolano(1.57 - acos((18000 - (pow(sqrt(fX * fX + fY * fY) - 25, 2) + fZ * fZ)) / 14400), bCzekaj);
			}
			else
				return -1;
		}
		else {
			if (fY > 0) {
				SetBiodroH(atan(-fX / fY), bCzekaj);
				SetBiodroV(atan((sqrt(fX * fX + fY * fY) - 25) / fZ) - acos((-10800 + fZ * fZ + pow(sqrt(fX * fX + fY * fY) - 25, 2)) / 120 / sqrt(fZ * fZ + pow(sqrt(fX * fX + fY * fY) - 25, 2))) + 1.57, bCzekaj);
				SetKolano(-1.57 + acos((18000 - (pow(sqrt(fX * fX + fY * fY) - 25, 2) + fZ * fZ)) / 14400), bCzekaj);
			}
			else
				return -1;
		}
		return 0;
	}
	//Zmiana poycji kocwki nogi robota we wsprzdnych nogi robota z moliwoci oczekiwania na wykonanie
	//fdX, fdY, fdZ [float] - zmiana wsprznych XYZ [mm]
	//bCzekaj [bool] - opcjonalne okrelenie czy ma oczekiwac na wykonanie polecenia (domylnie false)
	//zwraca [int] - 0 gdy OK, -1 gdy bd
	int AddPozycjaNogiAtNoga(float fdX, float fdY, float fdZ, bool bCzekaj = false) {
		float fXYZ[3];
		GetPozycjaNogiAtNoga(fXYZ, true);
		return SetPozycjaNogiAtNoga(fXYZ[0] + fdX, fXYZ[1] + fdY, fXYZ[2] + fdZ, bCzekaj);
	}
};

//Defineicja zmiennej identyfikujcej klienta symulacji dla wszystkich ng
int Noga::klientID=0;

//Klasa obsugujca robota
class Karaluch {
	//Wsprzdne robota w rodowisku
	int x, y, z;
	//Idntyfikator klienta symulacji
	int klientID = 0;
	//Identyfikator ciaa robota
	int KaraluchID = 0;
	//Identyfikator czjnika odlegoci
	int USSensorID = 0;
	//Identyfikator czjnika odlegoci
	int USSensorLewyID = 0;
	//Identyfikator czjnika odlegoci
	int USSensorPrawyID = 0;
	//Identyfikator pomiaru odlegoci
	int distanceID = 0;
public:
	//Zmienna i kostrukcja nogi prawy przd
	Noga* PP = new Noga(150, -90, 5);
	//Zmienna i kostrukcja nogi prawy rodek
	Noga* PS = new Noga(0, -90, 5);
	//Zmienna i kostrukcja nogi prawy ty
	Noga* PT = new Noga(-150, -90, 5);
	//Zmienna i kostrukcja nogi lewy przd
	Noga* LP = new Noga(150, 90, 5);
	//Zmienna i kostrukcja nogi lewy rodek
	Noga* LS = new Noga(0, 90, 5);
	//Zmienna i kostrukcja nogi lewy ty
	Noga* LT = new Noga(-150, 90, 5);
	//Konstruktor klasy Karaluch
	Karaluch() { Start(); }
	//konstruktor klasy Karaluch z numerem portu symulacji
	//iPort [int] - numer portu [int]
	Karaluch(int iPort) { Start((simxChar*)"127.0.0.1", iPort); }
	//konstruktor klasy Karaluch z definicj IP i numeru portu symulacji
	//cAdresIP [simxChar] - numer IP maszyny z dziaajc symulacj [string]
	//iPort [int] - numer portu [int]
	Karaluch(simxChar* cAdresIP, int iPort) { Start(cAdresIP, iPort); }
	//Sprawdzenie poprawnoci uruchomienia robota
	//zwraca [bool] - true gdy poczenie z symulacj zostao uruchomione poprawnie
	bool Uruchomione() { return klientID != -1; }
	//Uruchomienie poczenia z symulacj robota IP lokalne "127.0.0.1" i port nr 19000
	void Start() { Start((simxChar*)"127.0.0.1", 19000); }
	//Uruchomienie poczenia z symulacj robota z definicj IP oraz nr portu
	//cAdresIP [simxChar*] - adres IP komputer z uruchomion symulacj [string]
	//iPort [int] - umer portu [int]
	void Start(simxChar* cAdresIP, int iPort) {
		simxFinish(-1);                                                     //! Close any previously unfinished business
		klientID = simxStart(cAdresIP, iPort, true, true, 5000, 5);  //!< Main connection to V-REP
		Noga::klientID = klientID;
		Sleep(1);
		if (klientID != -1)
		{
			simxInt syncho = simxSynchronous(klientID, 1);
			int start = simxStartSimulation(klientID, simx_opmode_oneshot_wait);
			simxGetObjectHandle(klientID, "Cockroach", &KaraluchID, simx_opmode_oneshot_wait);
			simxGetObjectHandle(klientID, "SensorFront", &USSensorID, simx_opmode_oneshot_wait);
			simxGetObjectHandle(klientID, "SensorLeft", &USSensorLewyID, simx_opmode_oneshot_wait);
			simxGetObjectHandle(klientID, "SensorRight", &USSensorPrawyID, simx_opmode_oneshot_wait);
			simxGetDistanceHandle(klientID, "Distance", &distanceID, simx_opmode_oneshot_wait);
			simxGetObjectHandle(klientID, "Leg1HipH", &(PP->BiodroHID), simx_opmode_oneshot_wait);
			simxGetObjectHandle(klientID, "Leg1HipV", &(PP->BiodroVID), simx_opmode_oneshot_wait);
			simxGetObjectHandle(klientID, "Leg1Knee", &(PP->KolanoID), simx_opmode_oneshot_wait);
			simxGetObjectHandle(klientID, "Leg2HipH", &(PS->BiodroHID), simx_opmode_oneshot_wait);
			simxGetObjectHandle(klientID, "Leg2HipV", &(PS->BiodroVID), simx_opmode_oneshot_wait);
			simxGetObjectHandle(klientID, "Leg2Knee", &(PS->KolanoID), simx_opmode_oneshot_wait);
			simxGetObjectHandle(klientID, "Leg3HipH", &(PT->BiodroHID), simx_opmode_oneshot_wait);
			simxGetObjectHandle(klientID, "Leg3HipV", &(PT->BiodroVID), simx_opmode_oneshot_wait);
			simxGetObjectHandle(klientID, "Leg3Knee", &(PT->KolanoID), simx_opmode_oneshot_wait);
			simxGetObjectHandle(klientID, "Leg4HipH", &(LP->BiodroHID), simx_opmode_oneshot_wait);
			simxGetObjectHandle(klientID, "Leg4HipV", &(LP->BiodroVID), simx_opmode_oneshot_wait);
			simxGetObjectHandle(klientID, "Leg4Knee", &(LP->KolanoID), simx_opmode_oneshot_wait);
			simxGetObjectHandle(klientID, "Leg5HipH", &(LS->BiodroHID), simx_opmode_oneshot_wait);
			simxGetObjectHandle(klientID, "Leg5HipV", &(LS->BiodroVID), simx_opmode_oneshot_wait);
			simxGetObjectHandle(klientID, "Leg5Knee", &(LS->KolanoID), simx_opmode_oneshot_wait);
			simxGetObjectHandle(klientID, "Leg6HipH", &(LT->BiodroHID), simx_opmode_oneshot_wait);
			simxGetObjectHandle(klientID, "Leg6HipV", &(LT->BiodroVID), simx_opmode_oneshot_wait);
			simxGetObjectHandle(klientID, "Leg6Knee", &(LT->KolanoID), simx_opmode_oneshot_wait);
		}
	}
	//Pobranie identyfikatora klienta
	//zwraca [int] - numer ID klienta
	int GetClientID() { return klientID; }
	//Odczyt odlego od obiektu widzianego przez sensor
	//cJaki definicja sensora 'c' - centraly(domylny), 'l' - lewy, 'p' - prawy
	//zwraca [float] - odlego
	float GetDystans(char cJaki = 'c') {
		float fWartosc=0;
		simxUChar aa;
		simxInt intObjHandle;
		simxFloat sff;
		switch (cJaki) {
		case 'c':
			simxReadProximitySensor(klientID, USSensorID, &aa, &fWartosc, &intObjHandle, &sff, simx_opmode_oneshot_wait);
			break;
		case 'p':
			simxReadProximitySensor(klientID, USSensorPrawyID, &aa, &fWartosc, &intObjHandle, &sff, simx_opmode_oneshot_wait);
			break;
		case 'l':
			simxReadProximitySensor(klientID, USSensorLewyID, &aa, &fWartosc, &intObjHandle, &sff, simx_opmode_oneshot_wait);
			break;
		}
		if (aa != 0)
			simxReadDistance(klientID, distanceID, &sff, simx_opmode_oneshot_wait);
		else
			sff = -1;
		return sff;
	}
};
